@@ -1,6 +1,10 @@
-use crate::{DstImageView, SrcImageView};
+use crate::image_view::{TypedImageView, TypedImageViewMut};
+use crate::pixels::U8x4;
 
-pub(crate) fn multiply_alpha_native(src_image: &SrcImageView, dst_image: &mut DstImageView) {
+pub(crate) fn multiply_alpha_native(
+    src_image: TypedImageView<U8x4>,
+    mut dst_image: TypedImageViewMut<U8x4>,
+) {
     let src_rows = src_image.iter_rows(0, src_image.height().get());
     let dst_rows = dst_image.iter_rows_mut();
 
@@ -9,7 +13,7 @@ pub(crate) fn multiply_alpha_native(src_image: &SrcImageView, dst_image: &mut Ds
     }
 }
 
-pub(crate) fn multiply_alpha_inplace_native(image: &mut DstImageView) {
+pub(crate) fn multiply_alpha_inplace_native(mut image: TypedImageViewMut<U8x4>) {
     for dst_row in image.iter_rows_mut() {
         let src_row = unsafe { std::slice::from_raw_parts(dst_row.as_ptr(), dst_row.len()) };
         multiply_alpha_row_native(src_row, dst_row);
