@@ -6,17 +6,18 @@ use image::io::Reader as ImageReader;
 use image::{ColorType, GenericImageView};
 
 use fast_image_resize::{
-    CpuExtensions, FilterType, ImageData, PixelType, ResizeAlg, Resizer, SrcImageView,
+    CpuExtensions, DifferentTypesOfPixelsError, FilterType, Image, ImageView, PixelType, ResizeAlg,
+    Resizer,
 };
 
-fn get_source_image() -> ImageData<'static> {
+fn get_source_image_u8x4() -> Image<'static> {
     let img = ImageReader::open("./data/nasa-4928x3279.png")
         .unwrap()
         .decode()
         .unwrap();
     let width = img.width();
     let height = img.height();
-    ImageData::from_vec_u8(
+    Image::from_vec_u8(
         NonZeroU32::new(width).unwrap(),
         NonZeroU32::new(height).unwrap(),
         img.to_rgba8().into_raw(),
@@ -25,14 +26,30 @@ fn get_source_image() -> ImageData<'static> {
     .unwrap()
 }
 
-fn get_small_source_image() -> ImageData<'static> {
+fn get_source_image_u8x1() -> Image<'static> {
+    let img = ImageReader::open("./data/nasa-4928x3279.png")
+        .unwrap()
+        .decode()
+        .unwrap();
+    let width = img.width();
+    let height = img.height();
+    Image::from_vec_u8(
+        NonZeroU32::new(width).unwrap(),
+        NonZeroU32::new(height).unwrap(),
+        img.to_luma8().into_raw(),
+        PixelType::U8,
+    )
+    .unwrap()
+}
+
+fn get_small_source_image() -> Image<'static> {
     let img = ImageReader::open("./data/nasa-852x567.png")
         .unwrap()
         .decode()
         .unwrap();
     let width = img.width();
     let height = img.height();
-    ImageData::from_vec_u8(
+    Image::from_vec_u8(
         NonZeroU32::new(width).unwrap(),
         NonZeroU32::new(height).unwrap(),
         img.to_rgba8().into_raw(),
@@ -41,7 +58,7 @@ fn get_small_source_image() -> ImageData<'static> {
     .unwrap()
 }
 
-fn get_new_height(src_image: &SrcImageView, new_width: u32) -> u32 {
+fn get_new_height(src_image: &ImageView, new_width: u32) -> u32 {
     let scale = new_width as f32 / src_image.width().get() as f32;
     (src_image.height().get() as f32 * scale).round() as u32
 }
@@ -49,69 +66,79 @@ fn get_new_height(src_image: &SrcImageView, new_width: u32) -> u32 {
 const NEW_WIDTH: u32 = 255;
 const NEW_BIG_WIDTH: u32 = 5016;
 
-fn save_result(image: &SrcImageView, name: &str) {
+fn save_result(image: &Image, name: &str) {
     std::fs::create_dir_all("./data/result").unwrap();
     let mut file = File::create(format!("./data/result/{}.png", name)).unwrap();
-    let encoder = PngEncoder::new(&mut file);
-    encoder
+    let color_type = match image.pixel_type() {
+        PixelType::U8x4 => ColorType::Rgba8,
+        PixelType::U8 => ColorType::L8,
+        _ => panic!("Unsupported type of pixels"),
+    };
+    PngEncoder::new(&mut file)
         .encode(
-            &image.get_buffer(),
+            image.buffer(),
             image.width().get(),
             image.height().get(),
-            ColorType::Rgba8,
+            color_type,
         )
         .unwrap();
 }
 
 #[test]
 fn resize_wo_simd_lanczos3_test() {
-    let image = get_source_image();
+    let image = get_source_image_u8x4();
     let mut resizer = Resizer::new(ResizeAlg::Convolution(FilterType::Lanczos3));
     unsafe {
         resizer.set_cpu_extensions(CpuExtensions::None);
     }
-    let new_height = get_new_height(&image.src_view(), NEW_WIDTH);
-    let mut result = ImageData::new(
+    let new_height = get_new_height(&image.view(), NEW_WIDTH);
+    let mut result = Image::new(
         NonZeroU32::new(NEW_WIDTH).unwrap(),
         NonZeroU32::new(new_height).unwrap(),
         image.pixel_type(),
     );
-    resizer.resize(&image.src_view(), &mut result.dst_view());
-    save_result(&result.src_view(), "lanczos3_wo_simd");
+    assert!(resizer
+        .resize(&image.view(), &mut result.view_mut())
+        .is_ok());
+    save_result(&result, "u8x4-lanczos3-native");
 }
 
 #[test]
 fn resize_sse4_lanczos3_test() {
-    let image = get_source_image();
+    let image = get_source_image_u8x4();
     let mut resizer = Resizer::new(ResizeAlg::Convolution(FilterType::Lanczos3));
     unsafe {
         resizer.set_cpu_extensions(CpuExtensions::Sse4_1);
     }
-    let new_height = get_new_height(&image.src_view(), NEW_WIDTH);
-    let mut result = ImageData::new(
+    let new_height = get_new_height(&image.view(), NEW_WIDTH);
+    let mut result = Image::new(
         NonZeroU32::new(NEW_WIDTH).unwrap(),
         NonZeroU32::new(new_height).unwrap(),
         image.pixel_type(),
     );
-    resizer.resize(&image.src_view(), &mut result.dst_view());
-    save_result(&result.src_view(), "lanczos3_sse4");
+    assert!(resizer
+        .resize(&image.view(), &mut result.view_mut())
+        .is_ok());
+    save_result(&result, "u8x4-lanczos3-sse4");
 }
 
 #[test]
 fn resize_avx2_lanczos3_test() {
-    let image = get_source_image();
+    let image = get_source_image_u8x4();
     let mut resizer = Resizer::new(ResizeAlg::Convolution(FilterType::Lanczos3));
     unsafe {
         resizer.set_cpu_extensions(CpuExtensions::Avx2);
     }
-    let new_height = get_new_height(&image.src_view(), NEW_WIDTH);
-    let mut result = ImageData::new(
+    let new_height = get_new_height(&image.view(), NEW_WIDTH);
+    let mut result = Image::new(
         NonZeroU32::new(NEW_WIDTH).unwrap(),
         NonZeroU32::new(new_height).unwrap(),
         image.pixel_type(),
     );
-    resizer.resize(&image.src_view(), &mut result.dst_view());
-    save_result(&result.src_view(), "lanczos3_avx2");
+    assert!(resizer
+        .resize(&image.view(), &mut result.view_mut())
+        .is_ok());
+    save_result(&result, "u8x4-lanczos3-avx2");
 }
 
 #[test]
@@ -121,64 +148,109 @@ fn resize_avx2_lanczos3_upscale_test() {
     unsafe {
         resizer.set_cpu_extensions(CpuExtensions::Avx2);
     }
-    let new_height = get_new_height(&image.src_view(), NEW_BIG_WIDTH);
-    let mut result = ImageData::new(
+    let new_height = get_new_height(&image.view(), NEW_BIG_WIDTH);
+    let mut result = Image::new(
         NonZeroU32::new(NEW_BIG_WIDTH).unwrap(),
         NonZeroU32::new(new_height).unwrap(),
         image.pixel_type(),
     );
-    resizer.resize(&image.src_view(), &mut result.dst_view());
-    save_result(&result.src_view(), "lanczos3_avx2_upscale");
+    assert!(resizer
+        .resize(&image.view(), &mut result.view_mut())
+        .is_ok());
+    save_result(&result, "u8x4-lanczos3_upscale-avx2");
 }
 
 #[test]
 fn resize_nearest_test() {
-    let image = get_source_image();
+    let image = get_source_image_u8x4();
     let mut resizer = Resizer::new(ResizeAlg::Nearest);
     unsafe {
         resizer.set_cpu_extensions(CpuExtensions::None);
     }
-    let new_height = get_new_height(&image.src_view(), NEW_WIDTH);
-    let mut result = ImageData::new(
+    let new_height = get_new_height(&image.view(), NEW_WIDTH);
+    let mut result = Image::new(
         NonZeroU32::new(NEW_WIDTH).unwrap(),
         NonZeroU32::new(new_height).unwrap(),
         image.pixel_type(),
     );
-    resizer.resize(&image.src_view(), &mut result.dst_view());
-    save_result(&result.src_view(), "nearest_wo_simd");
+    assert!(resizer
+        .resize(&image.view(), &mut result.view_mut())
+        .is_ok());
+    save_result(&result, "u8x4-nearest-native");
 }
 
 #[test]
 fn resize_super_sampling_test() {
-    let image = get_source_image();
+    let image = get_source_image_u8x4();
     let mut resizer = Resizer::new(ResizeAlg::SuperSampling(FilterType::Lanczos3, 2));
     unsafe {
         resizer.set_cpu_extensions(CpuExtensions::Avx2);
     }
-    let new_height = get_new_height(&image.src_view(), NEW_WIDTH);
-    let mut result = ImageData::new(
+    let new_height = get_new_height(&image.view(), NEW_WIDTH);
+    let mut result = Image::new(
         NonZeroU32::new(NEW_WIDTH).unwrap(),
         NonZeroU32::new(new_height).unwrap(),
         image.pixel_type(),
     );
-    resizer.resize(&image.src_view(), &mut result.dst_view());
-    save_result(&result.src_view(), "super_sampling_avx2");
+    assert!(resizer
+        .resize(&image.view(), &mut result.view_mut())
+        .is_ok());
+    save_result(&result, "u8x4-super_sampling-avx2");
 }
 
 #[test]
-fn resize_with_cropping() {
-    let src_image = get_source_image();
+fn try_resize_to_other_pixel_type() {
+    let src_image = get_source_image_u8x4();
+    let mut resizer = Resizer::new(ResizeAlg::Convolution(FilterType::Lanczos3));
+    let mut dst_image = Image::new(
+        NonZeroU32::new(1024).unwrap(),
+        NonZeroU32::new(256).unwrap(),
+        PixelType::U8,
+    );
+    assert!(matches!(
+        resizer.resize(&src_image.view(), &mut dst_image.view_mut()),
+        Err(DifferentTypesOfPixelsError)
+    ));
+}
+
+#[test]
+fn resize_nearest_u8x1() {
+    let image = get_source_image_u8x1();
+    assert!(matches!(image.pixel_type(), PixelType::U8));
+
+    let mut resizer = Resizer::new(ResizeAlg::Nearest);
+    unsafe {
+        resizer.set_cpu_extensions(CpuExtensions::None);
+    }
+    let new_height = get_new_height(&image.view(), NEW_WIDTH);
+    let mut result = Image::new(
+        NonZeroU32::new(NEW_WIDTH).unwrap(),
+        NonZeroU32::new(new_height).unwrap(),
+        image.pixel_type(),
+    );
+    assert!(resizer
+        .resize(&image.view(), &mut result.view_mut())
+        .is_ok());
+    save_result(&result, "u8x1-nearest-native");
+}
+
+#[test]
+fn resize_lanczos3_u8x1() {
+    let image = get_source_image_u8x1();
+    assert!(matches!(image.pixel_type(), PixelType::U8));
+
     let mut resizer = Resizer::new(ResizeAlg::Convolution(FilterType::Lanczos3));
     unsafe {
         resizer.set_cpu_extensions(CpuExtensions::None);
     }
-    let mut dst_image = ImageData::new(
-        NonZeroU32::new(1024).unwrap(),
-        NonZeroU32::new(256).unwrap(),
-        src_image.pixel_type(),
+    let new_height = get_new_height(&image.view(), NEW_WIDTH);
+    let mut result = Image::new(
+        NonZeroU32::new(NEW_WIDTH).unwrap(),
+        NonZeroU32::new(new_height).unwrap(),
+        image.pixel_type(),
     );
-    let mut src_view = src_image.src_view();
-    src_view.set_crop_box_to_fit_dst_size(dst_image.width(), dst_image.height(), None);
-    resizer.resize(&src_view, &mut dst_image.dst_view());
-    save_result(&dst_image.src_view(), "cropping_lanczos3");
+    assert!(resizer
+        .resize(&image.view(), &mut result.view_mut())
+        .is_ok());
+    save_result(&result, "u8x1-lanczos3-native");
 }
